<?php

/**
 * Clear the xprofile_group cache when a new field is created, or when an existing field is deleted
 *
 * See: http://trac.buddypress.org/ticket/1332
 */
function nm_xprofile_clear_profile_group_cache($field) {
	$group_obj = (object) array( 'id' => $field->group_id );
	xprofile_clear_profile_groups_object_cache( $group_obj );
	wp_cache_delete( 'xprofile_group_' . $field->group_id, 'bp' );
}
add_action('xprofile_fields_saved_field', 'nm_xprofile_clear_profile_group_cache', 10, 1);
add_action('xprofile_fields_deleted_field', 'nm_xprofile_clear_profile_group_cache', 10, 1);

/**
 * Check the profile data cache is correct for a user, update it when necessary
 *
 * See: http://trac.buddypress.org/ticket/1332
 */
function nm_xprofile_group_fields($fields, $group_id) {
	global $bp, $profile_template;
	
	if ( is_a($profile_template, 'BP_XProfile_Data_Template') && isset($profile_template->group) ) {
		
		// Is there a difference?
		$diff = array_udiff($profile_template->group->fields, $fields, 'nm_profile_cache_diff');
		
		if ( ! empty($diff) ) {
		
			// The cached profile data is out-of-date, so replace it, and return the correct data
			// The fields defined by $profile_template->group->fields should be correct
			
			$fields = array();
			
			// Don't try and fetch any existing profile data if we are using this loop on the registration page
			$get_data = ( !bp_is_register_page() ) ? true : false;
			
			// Update the cache for the displayed user
			$user_id = $bp->displayed_user->id;
			
			for ( $i = 0; $i < count($profile_template->group->fields); $i++ ) {
		
				$field = new BP_XProfile_Field( $profile_template->group->fields[$i]->id, $user_id, $get_data );
				$fields[$i] = $field;
		
			}
		
			wp_cache_replace( 'xprofile_fields_' . $group_id . '_' . $user_id, $fields, 'bp' );
		
		}
	
	}
	
	return $fields;
}
add_filter('xprofile_group_fields', 'nm_xprofile_group_fields', 10, 2);

	/**
	 * Calculates the diff between the profile field cache and the profile data cache
	 * used by nm_xprofile_group_fields to determine whether we should update the
	 * profile data cache
	 */
	function nm_profile_cache_diff($a, $b) {
		$a = (object)$a;
		$b = (object)$b;
		if ($a->id == $b->id && $a->type == $b->type) {
			return 0;
		} elseif ($a->id > $b->id || $a->type > $b->type) {
			return 1;
		} else {
			return -1;
		}
	}
	
/**
 * Expire the profile data cache for a user after the profile is updated
 *
 * See: http://trac.buddypress.org/ticket/1332
 */
function nm_xprofile_data_after_save(&$profile_data) {
	global $bp, $wpdb;
	$sql = $wpdb->prepare("SELECT group_id FROM `{$bp->profile->table_name_fields}` WHERE id = %d", $profile_data->field_id);
	$group_id = $wpdb->get_var($sql);
	// Expire the cache
	wp_cache_delete( 'xprofile_fields_' . $group_id . '_' . $profile_data->user_id, 'bp' );
	return true;
}
add_action('xprofile_data_after_save', 'nm_xprofile_data_after_save');

?>